@@ -1,60 +1,60 @@
-""" Multicast DNS Service Discovery for Python, v0.14-wmcbrine
-    Copyright 2003 Paul Scott-Murphy, 2014 William McBrine
+"""Multicast DNS Service Discovery for Python, v0.14-wmcbrine
+Copyright 2003 Paul Scott-Murphy, 2014 William McBrine
 
-    This module provides a framework for the use of DNS Service Discovery
-    using IP multicast.
+This module provides a framework for the use of DNS Service Discovery
+using IP multicast.
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-    Lesser General Public License for more details.
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
-    USA
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
+USA
 """
 
 import asyncio
-import itertools
 import logging
-import random
-import socket
 import sys
 import threading
-from types import TracebackType  # noqa # used in type hints
-from typing import Any, Awaitable, Dict, List, Optional, Tuple, Type, Union, cast
+from types import TracebackType
+from typing import Awaitable, Dict, List, Optional, Set, Tuple, Type, Union
 
 from ._cache import DNSCache
 from ._dns import DNSQuestion, DNSQuestionType
+from ._engine import AsyncEngine
 from ._exceptions import NonUniqueNameException, NotRunningException
-from ._handlers import (
-    MulticastOutgoingQueue,
-    QueryHandler,
-    RecordManager,
-    construct_outgoing_multicast_answers,
-    construct_outgoing_unicast_answers,
-)
+from ._handlers.multicast_outgoing_queue import MulticastOutgoingQueue
+from ._handlers.query_handler import QueryHandler
+from ._handlers.record_manager import RecordManager
 from ._history import QuestionHistory
 from ._logger import QuietLogger, log
-from ._protocol.incoming import DNSIncoming
 from ._protocol.outgoing import DNSOutgoing
 from ._services import ServiceListener
 from ._services.browser import ServiceBrowser
-from ._services.info import ServiceInfo, instance_name_from_service_info
+from ._services.info import (
+    AsyncServiceInfo,
+    ServiceInfo,
+    instance_name_from_service_info,
+)
 from ._services.registry import ServiceRegistry
-from ._updates import RecordUpdate, RecordUpdateListener
+from ._transport import _WrappedTransport
+from ._updates import RecordUpdateListener
 from ._utils.asyncio import (
+    _resolve_all_futures_to_none,
     await_awaitable,
     get_running_loop,
     run_coro_with_timeout,
     shutdown_loop,
     wait_event_or_timeout,
+    wait_for_future_set_or_timeout,
 )
 from ._utils.name import service_type_name
 from ._utils.net import (
@@ -67,7 +67,6 @@ from ._utils.net import (
 )
 from ._utils.time import current_time_millis, millis_to_seconds
 from .const import (
-    _CACHE_CLEANUP_INTERVAL,
     _CHECK_TIME,
     _CLASS_IN,
     _CLASS_UNIQUE,
@@ -85,11 +84,10 @@ from .const import (
     _UNREGISTER_TIME,
 )
 
-_TC_DELAY_RANDOM_INTERVAL = (400, 500)
-# The maximum amont of time to delay a multicast
+# The maximum amount of time to delay a multicast
 # response in order to aggregate answers
 _AGGREGATION_DELAY = 500  # ms
-# The maximum amont of time to delay a multicast
+# The maximum amount of time to delay a multicast
 # response in order to aggregate answers after
 # it has already been delayed to protect the network
 # from excessive traffic. We use a shorter time
@@ -101,313 +99,9 @@ _AGGREGATION_DELAY = 500  # ms
 # 3000ms
 _PROTECTED_AGGREGATION_DELAY = 200  # ms
 
-_CLOSE_TIMEOUT = 3000  # ms
 _REGISTER_BROADCASTS = 3
 
 
-class _WrappedTransport:
-    """A wrapper for transports."""
-
-    __slots__ = (
-        'transport',
-        'is_ipv6',
-        'sock',
-        'fileno',
-        'sock_name',
-    )
-
-    def __init__(
-        self,
-        transport: asyncio.DatagramTransport,
-        is_ipv6: bool,
-        sock: socket.socket,
-        fileno: int,
-        sock_name: Any,
-    ) -> None:
-        """Initialize the wrapped transport.
-
-        These attributes are used when sending packets.
-        """
-        self.transport = transport
-        self.is_ipv6 = is_ipv6
-        self.sock = sock
-        self.fileno = fileno
-        self.sock_name = sock_name
-
-
-def _make_wrapped_transport(transport: asyncio.DatagramTransport) -> _WrappedTransport:
-    """Make a wrapped transport."""
-    sock: socket.socket = transport.get_extra_info('socket')
-    return _WrappedTransport(
-        transport=transport,
-        is_ipv6=sock.family == socket.AF_INET6,
-        sock=sock,
-        fileno=sock.fileno(),
-        sock_name=sock.getsockname(),
-    )
-
-
-class AsyncEngine:
-    """An engine wraps sockets in the event loop."""
-
-    def __init__(
-        self,
-        zeroconf: 'Zeroconf',
-        listen_socket: Optional[socket.socket],
-        respond_sockets: List[socket.socket],
-    ) -> None:
-        self.loop: Optional[asyncio.AbstractEventLoop] = None
-        self.zc = zeroconf
-        self.protocols: List[AsyncListener] = []
-        self.readers: List[_WrappedTransport] = []
-        self.senders: List[_WrappedTransport] = []
-        self.running_event: Optional[asyncio.Event] = None
-        self._listen_socket = listen_socket
-        self._respond_sockets = respond_sockets
-        self._cleanup_timer: Optional[asyncio.TimerHandle] = None
-
-    def setup(self, loop: asyncio.AbstractEventLoop, loop_thread_ready: Optional[threading.Event]) -> None:
-        """Set up the instance."""
-        self.loop = loop
-        self.running_event = asyncio.Event()
-        self.loop.create_task(self._async_setup(loop_thread_ready))
-
-    async def _async_setup(self, loop_thread_ready: Optional[threading.Event]) -> None:
-        """Set up the instance."""
-        assert self.loop is not None
-        self._cleanup_timer = self.loop.call_later(_CACHE_CLEANUP_INTERVAL, self._async_cache_cleanup)
-        await self._async_create_endpoints()
-        assert self.running_event is not None
-        self.running_event.set()
-        if loop_thread_ready:
-            loop_thread_ready.set()
-
-    async def _async_create_endpoints(self) -> None:
-        """Create endpoints to send and receive."""
-        assert self.loop is not None
-        loop = self.loop
-        reader_sockets = []
-        sender_sockets = []
-        if self._listen_socket:
-            reader_sockets.append(self._listen_socket)
-        for s in self._respond_sockets:
-            if s not in reader_sockets:
-                reader_sockets.append(s)
-            sender_sockets.append(s)
-
-        for s in reader_sockets:
-            transport, protocol = await loop.create_datagram_endpoint(lambda: AsyncListener(self.zc), sock=s)
-            self.protocols.append(cast(AsyncListener, protocol))
-            self.readers.append(_make_wrapped_transport(cast(asyncio.DatagramTransport, transport)))
-            if s in sender_sockets:
-                self.senders.append(_make_wrapped_transport(cast(asyncio.DatagramTransport, transport)))
-
-    def _async_cache_cleanup(self) -> None:
-        """Periodic cache cleanup."""
-        now = current_time_millis()
-        self.zc.question_history.async_expire(now)
-        self.zc.record_manager.async_updates(
-            now, [RecordUpdate(record, record) for record in self.zc.cache.async_expire(now)]
-        )
-        self.zc.record_manager.async_updates_complete(False)
-        assert self.loop is not None
-        self._cleanup_timer = self.loop.call_later(_CACHE_CLEANUP_INTERVAL, self._async_cache_cleanup)
-
-    async def _async_close(self) -> None:
-        """Cancel and wait for the cleanup task to finish."""
-        self._async_shutdown()
-        await asyncio.sleep(0)  # flush out any call soons
-        assert self._cleanup_timer is not None
-        self._cleanup_timer.cancel()
-
-    def _async_shutdown(self) -> None:
-        """Shutdown transports and sockets."""
-        assert self.running_event is not None
-        self.running_event.clear()
-        for wrapped_transport in itertools.chain(self.senders, self.readers):
-            wrapped_transport.transport.close()
-
-    def close(self) -> None:
-        """Close from sync context.
-
-        While it is not expected during normal operation,
-        this function may raise EventLoopBlocked if the underlying
-        call to `_async_close` cannot be completed.
-        """
-        assert self.loop is not None
-        # Guard against Zeroconf.close() being called from the eventloop
-        if get_running_loop() == self.loop:
-            self._async_shutdown()
-            return
-        if not self.loop.is_running():
-            return
-        run_coro_with_timeout(self._async_close(), self.loop, _CLOSE_TIMEOUT)
-
-
-class AsyncListener(asyncio.Protocol, QuietLogger):
-
-    """A Listener is used by this module to listen on the multicast
-    group to which DNS messages are sent, allowing the implementation
-    to cache information as it arrives.
-
-    It requires registration with an Engine object in order to have
-    the read() method called when a socket is available for reading."""
-
-    __slots__ = ('zc', 'data', 'last_time', 'transport', 'sock_description', '_deferred', '_timers')
-
-    def __init__(self, zc: 'Zeroconf') -> None:
-        self.zc = zc
-        self.data: Optional[bytes] = None
-        self.last_time: float = 0
-        self.transport: Optional[_WrappedTransport] = None
-        self.sock_description: Optional[str] = None
-        self._deferred: Dict[str, List[DNSIncoming]] = {}
-        self._timers: Dict[str, asyncio.TimerHandle] = {}
-        super().__init__()
-
-    def suppress_duplicate_packet(self, data: bytes, now: float) -> bool:
-        """Suppress duplicate packet if the last one was the same in the last second."""
-        if self.data == data and (now - 1000) < self.last_time:
-            return True
-        self.data = data
-        self.last_time = now
-        return False
-
-    def datagram_received(
-        self, data: bytes, addrs: Union[Tuple[str, int], Tuple[str, int, int, int]]
-    ) -> None:
-        assert self.transport is not None
-        v6_flow_scope: Union[Tuple[()], Tuple[int, int]] = ()
-        data_len = len(data)
-
-        if len(addrs) == 2:
-            # https://github.com/python/mypy/issues/1178
-            addr, port = addrs  # type: ignore
-            scope = None
-        else:
-            # https://github.com/python/mypy/issues/1178
-            addr, port, flow, scope = addrs  # type: ignore
-            log.debug('IPv6 scope_id %d associated to the receiving interface', scope)
-            v6_flow_scope = (flow, scope)
-
-        now = current_time_millis()
-        if self.suppress_duplicate_packet(data, now):
-            # Guard against duplicate packets
-            log.debug(
-                'Ignoring duplicate message received from %r:%r [socket %s] (%d bytes) as [%r]',
-                addr,
-                port,
-                self.sock_description,
-                data_len,
-                data,
-            )
-            return
-
-        if data_len > _MAX_MSG_ABSOLUTE:
-            # Guard against oversized packets to ensure bad implementations cannot overwhelm
-            # the system.
-            log.debug(
-                "Discarding incoming packet with length %s, which is larger "
-                "than the absolute maximum size of %s",
-                data_len,
-                _MAX_MSG_ABSOLUTE,
-            )
-            return
-
-        msg = DNSIncoming(data, (addr, port), scope, now)
-        if msg.valid:
-            log.debug(
-                'Received from %r:%r [socket %s]: %r (%d bytes) as [%r]',
-                addr,
-                port,
-                self.sock_description,
-                msg,
-                data_len,
-                data,
-            )
-        else:
-            log.debug(
-                'Received from %r:%r [socket %s]: (%d bytes) [%r]',
-                addr,
-                port,
-                self.sock_description,
-                data_len,
-                data,
-            )
-            return
-
-        if not msg.is_query():
-            self.zc.handle_response(msg)
-            return
-
-        self.handle_query_or_defer(msg, addr, port, self.transport, v6_flow_scope)
-
-    def handle_query_or_defer(
-        self,
-        msg: DNSIncoming,
-        addr: str,
-        port: int,
-        transport: _WrappedTransport,
-        v6_flow_scope: Union[Tuple[()], Tuple[int, int]] = (),
-    ) -> None:
-        """Deal with incoming query packets.  Provides a response if
-        possible."""
-        if not msg.truncated:
-            self._respond_query(msg, addr, port, transport, v6_flow_scope)
-            return
-
-        deferred = self._deferred.setdefault(addr, [])
-        # If we get the same packet we ignore it
-        for incoming in reversed(deferred):
-            if incoming.data == msg.data:
-                return
-        deferred.append(msg)
-        delay = millis_to_seconds(random.randint(*_TC_DELAY_RANDOM_INTERVAL))
-        assert self.zc.loop is not None
-        self._cancel_any_timers_for_addr(addr)
-        self._timers[addr] = self.zc.loop.call_later(
-            delay, self._respond_query, None, addr, port, transport, v6_flow_scope
-        )
-
-    def _cancel_any_timers_for_addr(self, addr: str) -> None:
-        """Cancel any future truncated packet timers for the address."""
-        if addr in self._timers:
-            self._timers.pop(addr).cancel()
-
-    def _respond_query(
-        self,
-        msg: Optional[DNSIncoming],
-        addr: str,
-        port: int,
-        transport: _WrappedTransport,
-        v6_flow_scope: Union[Tuple[()], Tuple[int, int]] = (),
-    ) -> None:
-        """Respond to a query and reassemble any truncated deferred packets."""
-        self._cancel_any_timers_for_addr(addr)
-        packets = self._deferred.pop(addr, [])
-        if msg:
-            packets.append(msg)
-
-        self.zc.handle_assembled_query(packets, addr, port, transport, v6_flow_scope)
-
-    def error_received(self, exc: Exception) -> None:
-        """Likely socket closed or IPv6."""
-        # We preformat the message string with the socket as we want
-        # log_exception_once to log a warrning message once PER EACH
-        # different socket in case there are problems with multiple
-        # sockets
-        msg_str = f"Error with socket {self.sock_description}): %s"
-        self.log_exception_once(exc, msg_str, exc)
-
-    def connection_made(self, transport: asyncio.BaseTransport) -> None:
-        wrapped_transport = _make_wrapped_transport(cast(asyncio.DatagramTransport, transport))
-        self.transport = wrapped_transport
-        self.sock_description = f"{wrapped_transport.fileno} ({wrapped_transport.sock_name})"
-
-    def connection_lost(self, exc: Optional[Exception]) -> None:
-        """Handle connection lost."""
-
-
 def async_send_with_transport(
     log_debug: bool,
     transport: _WrappedTransport,
@@ -427,7 +121,7 @@ def async_send_with_transport(
             return
     if log_debug:
         log.debug(
-            'Sending to (%s, %d) via [socket %s (%s)] (%d bytes #%d) %r as %r...',
+            "Sending to (%s, %d) via [socket %s (%s)] (%d bytes #%d) %r as %r...",
             real_addr,
             port or _MDNS_PORT,
             transport.fileno,
@@ -446,7 +140,6 @@ def async_send_with_transport(
 
 
 class Zeroconf(QuietLogger):
-
     """Implementation of Zeroconf Multicast DNS Service Discovery
 
     Supports registration, unregistration, queries and browsing.
@@ -479,12 +172,12 @@ class Zeroconf(QuietLogger):
 
         self.done = False
 
-        if apple_p2p and sys.platform != 'darwin':
-            raise RuntimeError('Option `apple_p2p` is not supported on non-Apple platforms.')
+        if apple_p2p and sys.platform != "darwin":
+            raise RuntimeError("Option `apple_p2p` is not supported on non-Apple platforms.")
 
         self.unicast = unicast
         listen_socket, respond_sockets = create_sockets(interfaces, unicast, ip_version, apple_p2p=apple_p2p)
-        log.debug('Listen socket %s, respond sockets %s', listen_socket, respond_sockets)
+        log.debug("Listen socket %s, respond sockets %s", listen_socket, respond_sockets)
 
         self.engine = AsyncEngine(self, listen_socket, respond_sockets)
 
@@ -492,16 +185,17 @@ class Zeroconf(QuietLogger):
         self.registry = ServiceRegistry()
         self.cache = DNSCache()
         self.question_history = QuestionHistory()
-        self.query_handler = QueryHandler(self.registry, self.cache, self.question_history)
+
+        self.out_queue = MulticastOutgoingQueue(self, 0, _AGGREGATION_DELAY)
+        self.out_delay_queue = MulticastOutgoingQueue(self, _ONE_SECOND, _PROTECTED_AGGREGATION_DELAY)
+
+        self.query_handler = QueryHandler(self)
         self.record_manager = RecordManager(self)
 
-        self.notify_event: Optional[asyncio.Event] = None
+        self._notify_futures: Set[asyncio.Future] = set()
         self.loop: Optional[asyncio.AbstractEventLoop] = None
         self._loop_thread: Optional[threading.Thread] = None
 
-        self._out_queue = MulticastOutgoingQueue(self, 0, _AGGREGATION_DELAY)
-        self._out_delay_queue = MulticastOutgoingQueue(self, _ONE_SECOND, _PROTECTED_AGGREGATION_DELAY)
-
         self.start()
 
     @property
@@ -513,7 +207,6 @@ class Zeroconf(QuietLogger):
         """Start Zeroconf."""
         self.loop = get_running_loop()
         if self.loop:
-            self.notify_event = asyncio.Event()
             self.engine.setup(self.loop, None)
             return
         self._start_thread()
@@ -525,7 +218,6 @@ class Zeroconf(QuietLogger):
         def _run_loop() -> None:
             self.loop = asyncio.new_event_loop()
             asyncio.set_event_loop(self.loop)
-            self.notify_event = asyncio.Event()
             self.engine.setup(self.loop, loop_thread_ready)
             self.loop.run_forever()
 
@@ -547,13 +239,14 @@ class Zeroconf(QuietLogger):
             raise NotRunningException
 
     @property
-    def listeners(self) -> List[RecordUpdateListener]:
+    def listeners(self) -> Set[RecordUpdateListener]:
         return self.record_manager.listeners
 
     async def async_wait(self, timeout: float) -> None:
         """Calling task waits for a given number of milliseconds or until notified."""
-        assert self.notify_event is not None
-        await wait_event_or_timeout(self.notify_event, timeout=millis_to_seconds(timeout))
+        loop = self.loop
+        assert loop is not None
+        await wait_for_future_set_or_timeout(loop, self._notify_futures, timeout)
 
     def notify_all(self) -> None:
         """Notifies all waiting threads and notify listeners."""
@@ -562,16 +255,26 @@ class Zeroconf(QuietLogger):
 
     def async_notify_all(self) -> None:
         """Schedule an async_notify_all."""
-        assert self.notify_event is not None
-        self.notify_event.set()
-        self.notify_event.clear()
+        notify_futures = self._notify_futures
+        if notify_futures:
+            _resolve_all_futures_to_none(notify_futures)
 
     def get_service_info(
-        self, type_: str, name: str, timeout: int = 3000, question_type: Optional[DNSQuestionType] = None
+        self,
+        type_: str,
+        name: str,
+        timeout: int = 3000,
+        question_type: Optional[DNSQuestionType] = None,
     ) -> Optional[ServiceInfo]:
         """Returns network's service information for a particular
         name and type, or None if no service matches by the timeout,
-        which defaults to 3 seconds."""
+        which defaults to 3 seconds.
+
+        :param type_: fully qualified service type name
+        :param name: the name of the service
+        :param timeout: milliseconds to wait for a response
+        :param question_type: The type of questions to ask (DNSQuestionType.QM or DNSQuestionType.QU)
+        """
         info = ServiceInfo(type_, name)
         if info.request(self, timeout, question_type):
             return info
@@ -601,6 +304,7 @@ class Zeroconf(QuietLogger):
         ttl: Optional[int] = None,
         allow_name_change: bool = False,
         cooperating_responders: bool = False,
+        strict: bool = True,
     ) -> None:
         """Registers service information to the network with a default TTL.
         Zeroconf will then respond to requests for information for that
@@ -616,7 +320,7 @@ class Zeroconf(QuietLogger):
         assert self.loop is not None
         run_coro_with_timeout(
             await_awaitable(
-                self.async_register_service(info, ttl, allow_name_change, cooperating_responders)
+                self.async_register_service(info, ttl, allow_name_change, cooperating_responders, strict)
             ),
             self.loop,
             _REGISTER_TIME * _REGISTER_BROADCASTS,
@@ -628,6 +332,7 @@ class Zeroconf(QuietLogger):
         ttl: Optional[int] = None,
         allow_name_change: bool = False,
         cooperating_responders: bool = False,
+        strict: bool = True,
     ) -> Awaitable:
         """Registers service information to the network with a default TTL.
         Zeroconf will then respond to requests for information for that
@@ -643,7 +348,7 @@ class Zeroconf(QuietLogger):
 
         info.set_server_if_missing()
         await self.async_wait_for_start()
-        await self.async_check_service(info, allow_name_change, cooperating_responders)
+        await self.async_check_service(info, allow_name_change, cooperating_responders, strict)
         self.registry.async_add(info)
         return asyncio.ensure_future(self._async_broadcast_service(info, _REGISTER_TIME, None))
 
@@ -658,7 +363,9 @@ class Zeroconf(QuietLogger):
         """
         assert self.loop is not None
         run_coro_with_timeout(
-            await_awaitable(self.async_update_service(info)), self.loop, _REGISTER_TIME * _REGISTER_BROADCASTS
+            await_awaitable(self.async_update_service(info)),
+            self.loop,
+            _REGISTER_TIME * _REGISTER_BROADCASTS,
         )
 
     async def async_update_service(self, info: ServiceInfo) -> Awaitable:
@@ -668,6 +375,27 @@ class Zeroconf(QuietLogger):
         self.registry.async_update(info)
         return asyncio.ensure_future(self._async_broadcast_service(info, _REGISTER_TIME, None))
 
+    async def async_get_service_info(
+        self,
+        type_: str,
+        name: str,
+        timeout: int = 3000,
+        question_type: Optional[DNSQuestionType] = None,
+    ) -> Optional[AsyncServiceInfo]:
+        """Returns network's service information for a particular
+        name and type, or None if no service matches by the timeout,
+        which defaults to 3 seconds.
+
+        :param type_: fully qualified service type name
+        :param name: the name of the service
+        :param timeout: milliseconds to wait for a response
+        :param question_type: The type of questions to ask (DNSQuestionType.QM or DNSQuestionType.QU)
+        """
+        info = AsyncServiceInfo(type_, name)
+        if await info.async_request(self, timeout, question_type):
+            return info
+        return None
+
     async def _async_broadcast_service(
         self,
         info: ServiceInfo,
@@ -704,7 +432,7 @@ class Zeroconf(QuietLogger):
         #
         # _CLASS_UNIQUE is the "QU" bit
         out.add_question(DNSQuestion(info.type, _TYPE_PTR, _CLASS_IN | _CLASS_UNIQUE))
-        out.add_authorative_answer(info.dns_pointer(created=current_time_millis()))
+        out.add_authorative_answer(info.dns_pointer())
         return out
 
     def _add_broadcast_answer(  # pylint: disable=no-self-use
@@ -715,15 +443,15 @@ class Zeroconf(QuietLogger):
         broadcast_addresses: bool = True,
     ) -> None:
         """Add answers to broadcast a service."""
-        now = current_time_millis()
-        other_ttl = info.other_ttl if override_ttl is None else override_ttl
-        host_ttl = info.host_ttl if override_ttl is None else override_ttl
-        out.add_answer_at_time(info.dns_pointer(override_ttl=other_ttl, created=now), 0)
-        out.add_answer_at_time(info.dns_service(override_ttl=host_ttl, created=now), 0)
-        out.add_answer_at_time(info.dns_text(override_ttl=other_ttl, created=now), 0)
+        current_time_millis()
+        other_ttl = None if override_ttl is None else override_ttl
+        host_ttl = None if override_ttl is None else override_ttl
+        out.add_answer_at_time(info.dns_pointer(override_ttl=other_ttl), 0)
+        out.add_answer_at_time(info.dns_service(override_ttl=host_ttl), 0)
+        out.add_answer_at_time(info.dns_text(override_ttl=other_ttl), 0)
         if broadcast_addresses:
-            for dns_address in info.dns_addresses(override_ttl=host_ttl, created=now):
-                out.add_answer_at_time(dns_address, 0)
+            for record in info.get_address_and_nsec_records(override_ttl=host_ttl):
+                out.add_answer_at_time(record, 0)
 
     def unregister_service(self, info: ServiceInfo) -> None:
         """Unregister a service.
@@ -734,7 +462,9 @@ class Zeroconf(QuietLogger):
         """
         assert self.loop is not None
         run_coro_with_timeout(
-            self.async_unregister_service(info), self.loop, _UNREGISTER_TIME * _REGISTER_BROADCASTS
+            self.async_unregister_service(info),
+            self.loop,
+            _UNREGISTER_TIME * _REGISTER_BROADCASTS,
         )
 
     async def async_unregister_service(self, info: ServiceInfo) -> Awaitable:
@@ -744,8 +474,8 @@ class Zeroconf(QuietLogger):
         # If another server uses the same addresses, we do not want to send
         # goodbye packets for the address records
 
-        assert info.server is not None
-        entries = self.registry.async_get_infos_server(info.server.lower())
+        assert info.server_key is not None
+        entries = self.registry.async_get_infos_server(info.server_key)
         broadcast_addresses = not bool(entries)
         return asyncio.ensure_future(
             self._async_broadcast_service(info, _UNREGISTER_TIME, 0, broadcast_addresses)
@@ -787,15 +517,21 @@ class Zeroconf(QuietLogger):
         """
         assert self.loop is not None
         run_coro_with_timeout(
-            self.async_unregister_all_services(), self.loop, _UNREGISTER_TIME * _REGISTER_BROADCASTS
+            self.async_unregister_all_services(),
+            self.loop,
+            _UNREGISTER_TIME * _REGISTER_BROADCASTS,
         )
 
     async def async_check_service(
-        self, info: ServiceInfo, allow_name_change: bool, cooperating_responders: bool = False
+        self,
+        info: ServiceInfo,
+        allow_name_change: bool,
+        cooperating_responders: bool = False,
+        strict: bool = True,
     ) -> None:
         """Checks the network for a unique service name, modifying the
         ServiceInfo passed in if it is not unique."""
-        instance_name = instance_name_from_service_info(info)
+        instance_name = instance_name_from_service_info(info, strict=strict)
         if cooperating_responders:
             return
         next_instance_number = 2
@@ -808,9 +544,9 @@ class Zeroconf(QuietLogger):
                     raise NonUniqueNameException
 
                 # change the name and look for a conflict
-                info.name = f'{instance_name}-{next_instance_number}.{info.type}'
+                info.name = f"{instance_name}-{next_instance_number}.{info.type}"
                 next_instance_number += 1
-                service_type_name(info.name)
+                service_type_name(info.name, strict=strict)
                 next_time = now
                 i = 0
 
@@ -824,7 +560,9 @@ class Zeroconf(QuietLogger):
             next_time += _CHECK_TIME
 
     def add_listener(
-        self, listener: RecordUpdateListener, question: Optional[Union[DNSQuestion, List[DNSQuestion]]]
+        self,
+        listener: RecordUpdateListener,
+        question: Optional[Union[DNSQuestion, List[DNSQuestion]]],
     ) -> None:
         """Adds a listener for a given question.  The listener will have
         its update_record method called when information is available to
@@ -844,7 +582,9 @@ class Zeroconf(QuietLogger):
         self.loop.call_soon_threadsafe(self.record_manager.async_remove_listener, listener)
 
     def async_add_listener(
-        self, listener: RecordUpdateListener, question: Optional[Union[DNSQuestion, List[DNSQuestion]]]
+        self,
+        listener: RecordUpdateListener,
+        question: Optional[Union[DNSQuestion, List[DNSQuestion]]],
     ) -> None:
         """Adds a listener for a given question.  The listener will have
         its update_record method called when information is available to
@@ -861,48 +601,6 @@ class Zeroconf(QuietLogger):
         """
         self.record_manager.async_remove_listener(listener)
 
-    def handle_response(self, msg: DNSIncoming) -> None:
-        """Deal with incoming response packets.  All answers
-        are held in the cache, and listeners are notified."""
-        self.record_manager.async_updates_from_response(msg)
-
-    def handle_assembled_query(
-        self,
-        packets: List[DNSIncoming],
-        addr: str,
-        port: int,
-        transport: _WrappedTransport,
-        v6_flow_scope: Union[Tuple[()], Tuple[int, int]] = (),
-    ) -> None:
-        """Respond to a (re)assembled query.
-
-        If the protocol recieved packets with the TC bit set, it will
-        wait a bit for the rest of the packets and only call
-        handle_assembled_query once it has a complete set of packets
-        or the timer expires. If the TC bit is not set, a single
-        packet will be in packets.
-        """
-        now = packets[0].now
-        ucast_source = port != _MDNS_PORT
-        question_answers = self.query_handler.async_response(packets, ucast_source)
-        if question_answers.ucast:
-            questions = packets[0].questions
-            id_ = packets[0].id
-            out = construct_outgoing_unicast_answers(question_answers.ucast, ucast_source, questions, id_)
-            # When sending unicast, only send back the reply
-            # via the same socket that it was recieved from
-            # as we know its reachable from that socket
-            self.async_send(out, addr, port, v6_flow_scope, transport)
-        if question_answers.mcast_now:
-            self.async_send(construct_outgoing_multicast_answers(question_answers.mcast_now))
-        if question_answers.mcast_aggregate:
-            self._out_queue.async_add(now, question_answers.mcast_aggregate)
-        if question_answers.mcast_aggregate_last_second:
-            # https://datatracker.ietf.org/doc/html/rfc6762#section-14
-            # If we broadcast it in the last second, we have to delay
-            # at least a second before we send it again
-            self._out_delay_queue.async_add(now, question_answers.mcast_aggregate_last_second)
-
     def send(
         self,
         out: DNSOutgoing,
@@ -934,11 +632,23 @@ class Zeroconf(QuietLogger):
 
         for packet_num, packet in enumerate(out.packets()):
             if len(packet) > _MAX_MSG_ABSOLUTE:
-                self.log_warning_once("Dropping %r over-sized packet (%d bytes) %r", out, len(packet), packet)
+                self.log_warning_once(
+                    "Dropping %r over-sized packet (%d bytes) %r",
+                    out,
+                    len(packet),
+                    packet,
+                )
                 return
             for send_transport in transports:
                 async_send_with_transport(
-                    log_debug, send_transport, packet, packet_num, out, addr, port, v6_flow_scope
+                    log_debug,
+                    send_transport,
+                    packet,
+                    packet_num,
+                    out,
+                    addr,
+                    port,
+                    v6_flow_scope,
                 )
 
     def _close(self) -> None:
@@ -991,7 +701,7 @@ class Zeroconf(QuietLogger):
         await self.engine._async_close()  # pylint: disable=protected-access
         self._shutdown_threads()
 
-    def __enter__(self) -> 'Zeroconf':
+    def __enter__(self) -> "Zeroconf":
         return self
 
     def __exit__(  # pylint: disable=useless-return
